%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This is part of the set of files that accompany the article:       %
%                                                                   %
%Ricardo Reis (2008) "Estimating a SIGE model for policy evaluation"%
%in Monetary Policy under Uncertainty, Central Bank of Chile.       %
%                                                                   %
%See the appendix of the paper or its NBER or CEPR working paper    %
%versions for a detailed explanation of the algorithms.             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Please cite if you use the programs. I do not provide tech support.%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This file: mix the draws form several chains and assess convergence%
%Last revised: October 30, 2007                                     %
%Written by: Tiago Berriel and Ricardo Reis                         %
%Input: The draws in BayesdrawsGREEN0x.mat,                         %
%Output: Printout of convergence diagnostics and Chile_GREENpost.mat%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear; clc; clear global

%%%%% STEP 1: LOAD FILES %%%%%%
load BayesdrawUSG01 draws accepts logds;
FinalDraws=draws'; FinalAccepts=accepts; FinalLogds=logds;
load BayesdrawUSG02 draws accepts logds;
FinalDraws=[FinalDraws draws']; FinalAccepts=[FinalAccepts accepts];
FinalLogds=[FinalLogds logds];
load BayesdrawUSG03 draws accepts logds;
FinalDraws=[FinalDraws draws']; FinalAccepts=[FinalAccepts accepts];
FinalLogds=[FinalLogds logds];
load BayesdrawUSG04 draws accepts logds;
FinalDraws=[FinalDraws draws']; FinalAccepts=[FinalAccepts accepts];
FinalLogds=[FinalLogds logds];
load BayesdrawUSG05 draws accepts logds;
FinalDraws=[FinalDraws draws']; FinalAccepts=[FinalAccepts accepts];
FinalLogds=[FinalLogds logds];
clear accept accepts draws logds

%%%%% STEP 2: DISPLAYING RESULTS AND DETERMINING BURN-IN %%%%%%
load BayesdrawUSG01 pars_c; load SIGE_USGmode x0 VARCOV;
disp('Calibrated parameters:');
disp('    beta      psi       theta    nu        gamma      phiy      phipi')
disp(pars_c(:,1:7));
disp('    rho_m     sigma_m   rho_a     sigma_a');
disp(pars_c(:,8:11));
disp('    rho_g     sigma_g   rho_nu    sigma_nu  rho_gam   sigma_gam');
disp(pars_c(:,12:17));
disp('delta omega lambda');
disp(pars_c(:,18:20));
disp(' ')
disp('Total number of draws and number of independent samples:');
[M m]=size(FinalAccepts); disp([M m]);
kk=16; disp('Number of parameters estimated:'); disp(kk);
burnin=100000; disp('Burn-in draws:'); disp(burnin);
accept_rate = mean(sum(FinalAccepts(burnin+1:M,:))./(M-burnin));
disp('Accept rates:'); disp(accept_rate);
results = FinalDraws(burnin+1:M,:); reslogds = FinalLogds(burnin+1:M,:);

%%%%% STEP 3: ASSESSING CONVERGENCE OF CHAIN %%%%%%
SIGEconverge(results,kk,10);

%%%%% STEP 4: MIX THE DRAWS %%%%%%
MixedResults=[]; MixedLogds=[];
for i=1:m; MixedResults=[MixedResults; results(:,kk*(i-1)+1:kk*i)];
    MixedLogds=[MixedLogds; reslogds(:,m)]; end
Md = length(MixedResults);  %length of mixed sample
mod=x0'; mod_se=sqrt(diag(VARCOV))'; %posterior mode
mea=mean(MixedResults); med=median(MixedResults); %posterior mean and median
vc=cov(MixedResults); se=std(MixedResults); %posterior var-cov and its diagonal
lb = prctile(MixedResults,2.5); ub = prctile(MixedResults,97.5); %posterior percentiles
clear Pmean Ppar Pvar VARCOV Ri Vi Wi VARCOV i m xo FinalDraws
clear FinalLogds FinalAccepts results x0 M reslogds

%%%%% STEP 5: DISPLAY THE DRAWS %%%%%%
%Mode
disp('Bayesian mode parameters and Hessian:  ')
disp('      nu      gamma      phiy     phipi    rho_m     sigma_m')
disp([mod(1:6);mod_se(1:6)]);
disp('    rho_g     sigma_g   rho_nu    sigma_nu  rho_gam   sigma_gam')
disp([mod(7:12);mod_se(7:12)]);
disp('    delta     omega     lambda  psi');
disp([mod(13:kk);mod_se(13:kk)]);
%Mean
disp('Bayesian mean parameters and standard errors:  ')
disp('      nu      gamma      phiy     phipi    rho_m     sigma_m')
disp([mea(1:6);se(1:6)]);
disp('    rho_g     sigma_g   rho_nu    sigma_nu  rho_gam   sigma_gam')
disp([mea(7:12);se(7:12)]);
disp('    delta     omega     lambda  psi');
disp([mea(13:kk);se(13:kk)]);
%Median
disp('Bayesian median parameters and 95% C.I.:');
disp('      nu      gamma      phiy     phipi    rho_m     sigma_m')
disp([med(1:6);lb(1:6);ub(1:6)]);
disp('    rho_g     sigma_g   rho_nu    sigma_nu  rho_gam   sigma_gam')
disp([med(7:12);lb(7:12);ub(7:12)]);
disp('    delta     omega     lambda  psi');
disp([med(13:kk);lb(13:kk);ub(13:kk)]);
%Histograms
disp('See figure for histograms');
figure; for i=1:6; subplot(6,1,i), hist(MixedResults(:,i),50),...
        title('First set of parameters'); end
figure; for i=7:12; subplot(12-6,1,i-6), hist(MixedResults(:,i),50),...
        title('Second set of parameters'); end
figure; for i=13:kk; subplot(kk-12,1,i-12), hist(MixedResults(:,i),50),...
        title('Third set of parameters'); end

%%%%% STEP 6: SAVE RESULTS IN USEFUL FORMAT FOR POLICY ANALYSIS%%%%%%
clear R i mod_se accept_rate burnin kk vc ans cc lb ub se
%pile all in 1x20 vector including estimates and calibrations
est_median=pars_c; est_median(1,4:9)=med(1,1:6); ...
    est_median(1,12:20)=med(1,7:15); est_median(1,2)=med(1,16);
est_mean=pars_c; est_mean(1,4:9)=mea(1,1:6); ...
    est_mean(1,12:20)=mea(1,7:15); est_mean(1,2)=mea(1,16);
est_mode=pars_c; est_mode(1,4:9)=mod(1,1:6); ...
    est_mode(1,12:20)=mod(1,7:15); est_mode(1,2)=mod(1,16);
ests = ones(Md,1)*pars_c;
ests(:,4:9)=MixedResults(:,1:6); ests(:,12:20)=MixedResults(:,7:15); ...
    ests(:,2)=MixedResults(:,16);
save SIGE_USGpost.mat