%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This is part of the set of files that accompany the article:       %
%                                                                   %
%Ricardo Reis (2008) "Estimating a SIGE model for policy evaluation"%
%in Monetary Policy under Uncertainty, Central Bank of Chile.       %
%                                                                   %
%See the appendix of the paper or its NBER or CEPR working paper    %
%versions for a detailed explanation of the algorithms.             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Please cite if you use the programs. I do not provide tech support.%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This file: find mode of posterior density                          %
%Last revised: October 15, 2007                                     %
%Written by: Tiago Berriel and Ricardo Reis                         %
%Input: The data in USGdata.mat. Chile_USGmode_run with searches  %
%Output: The mode that will be used, Chile_USGmode                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%STEP 1: LOADING DATA AND MANIPULATING IT%%%%%
clear; clc; clear global;
load USGdata.mat; global T K; [T K]=size(data); %load data
% Data matrix is Txk, where T is the number of observations and K is the
%number of variables in the model. First column is Dp, second is Dy, third
%is i, fourth is Dwp, fifth is l. All already de-meaned.
global Y; Y=data(1,1:K)'; for i=2:T; Y=[Y;data(i,1:K)']; end;
% Y matrix is 5*Tx1 and has the observations stacked
global N; N=100;     %periods in MA representations

%%%%STEP 2: PARAMETER INPUTS - CALIBRATED PARAMETERS %%%%%%%
beta = 2/3;       %variable labor share
theta = 1;        %elasticity of intertemporal substitution
Da = data(2:T,2) - beta*(data(2:T,5)-data(1:T-1,5));
[rho_a,temp1,temp2,temp3,temp4] = regress(Da(2:T-1,1),Da(1:T-2,1));
sigma_a = sqrt(temp4(4)); %rho_a and sigma_a
clear Da temp1 temp2 temp3 temp4
global pars_c;
pars_c=[beta NaN theta NaN NaN NaN NaN NaN NaN rho_a sigma_a NaN];
pars_c=[pars_c NaN NaN NaN NaN NaN NaN NaN NaN];

%%%%% STEP 3: SETTING BOUNDS AND STARTING VALUE %%%%%%
%lb has lower bound, ub has upper ound, x0 has initial guess
%  [ nu    gamma   phiy   phipi   rho_e   sig_e ]
lb=[1.001; 1.001; 0.0001; 1.0001; 0.01 ; 0.00001];
ub=[100  ; 100  ; 10    ; 10    ; 0.99 ;   100  ];
%  [   rho_g  sig_g   rho_nu  sig_nu  rho_gam sigma_gam]
lb=[lb;0.01; 0.00001;  0.01; 0.00001;  0.01  ; 0.00001 ];
ub=[ub;0.99;   100  ;  0.99;   100  ;  0.99  ;   100   ];
%  [   delta  omega  lambda]
lb=[lb; 0.01; 0.01;  0.01  ];
ub=[ub; 0.98; 0.98;  0.98  ];
%  [    psi ]
lb=[lb; 0.01];
ub=[ub; 20  ];
%Starting value
%If have previous searches:
%load SIGE_USGmode_run Result1 Result2;
%[a b]=min(Result2); x0=Result1(b,:)'; clear a b Result1 Result2;
%Otherwise set one
x0 = [6.8336	9.2155	0.0348	1.0001	0.5556	0.72	0.9448  1.56 ...
      0.6105	28.38	0.4426	87.77	0.3838	0.1137	0.253   4]';


%%%%% STEP 4: MINIMISE MINUS POSTERIOR TO FIND MODE %%%%%%
tic
options = optimset('MaxFunEvals',100000);
%x0 = fminsearch(@SIGEpost,x0); %first look line search
[Mode fval a b c d hess] = ...
    fmincon(@SIGEpost,x0,[],[],[],[],lb,ub,[],options); clear a b c d
    %now back to fmincon to get the Hessian
VARCOV = inv(hess); SE = diag(VARCOV); SE = sqrt(SE);
toc

%%%%% STEP 4: DISPLAYING AND SAVING %%%%%%
disp('##########################################################')
disp('Parameters:  ')
disp('    nu        gamma     phiy      phipi    rho_g     sigma_g   rho_nu    sigma_nu  rho_gam   sigma_gam delta     omega     lambda   psi');
disp(Mode')
disp('Standard errors:  '); disp(SE')
disp('Posterior:'); disp(fval)
disp('Variance-covariance matrix:'); disp(VARCOV)
disp('##########################################################')
clear K N T Y a b beta c d data i lb options phipi phiy psi rho_a rho_m
clear sigma_a sigma_m theta ub
x0=Mode;
VARCOV=ImposeSymmetry(VARCOV); mvnrnd(x0,VARCOV); %imposing symmetry
save SIGE_USGmode x0 VARCOV