function [Rp maxR] = SIGEconverge(draws,kk,bat)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This is part of the set of files that accompany the article:       %
%Ricardo Reis (2008) "Estimating a SIGE model for policy evaluation"%
%in Monetary Policy under Uncertainty, Central Bank of Chile.       %
%                                                                   %
%It updates SIGEconverge to take into acount any number of          %
%parameters. It really follows the book, pages 296, 297, where      %
%quantities of interest are the estimates of each parameter.        %
%(see Brooks-Gellman, 1998, page 45.)                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Please cite if you use the programs. I do not provide tech support.%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Last revised: January15, 2006                                      %
%Written by: Ricardo Reis                                           %
%Input: draws, number of batches to split, number of parameters     %
%Output: Displayed, as well as reporting multivariate and scalar Rs %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%bat is the number of batches I want to break samples in
%kk is the the number of parameters being estimated
[n temp]=size(draws); %n is the length (number of draws) in each chain
b=n/bat;   %b is the size of each batch once break them
m=temp/kk; %m is the number of independent chains that I ran
disp('***************CONVERGENCE DIAGNOSTICS*******************')

%STEP 1: CALCULATE R STATISTIC USING ONLY THE ORIGINAL NUMBER OF BATCHES I HAVE
s2 = zeros(kk,kk);  %just to initialise it
for i=1:m; %across the chains
    series = draws(:,kk*(i-1)+1:kk*i); %extract each chain in turn
    s2 = s2 + cov(series);   %calculate s2.
    mu(i,:)=mean(series);    %mean estimate of each simulation, stacked
end
%BG notation but with i added for GCSR notation
W = s2/m; Bn = cov(mu); Wi = diag(W)'; Bni = diag(Bn)';
V = ((n-1)/n)*W + (1+1/m)*Bn; Vi = ((n-1)/n)*Wi + Bni;
Rp = ((n-1)/n) + ((m+1)/m)*max(eig(inv(W)*V));
Ri = sqrt(Vi./Wi); maxR=max(Ri); neff = m*Vi./Bni;
disp(' Individual scale factors on whole sample:'); disp(Ri); disp(' ');
disp(' Largest individual scale factor:'); disp(maxR); disp(' ');
disp(' Effective number of independent draws:'); disp(neff); disp(' ');
disp(' R_p statistic:'); disp(Rp); disp(' ');

%STEP 2: CALCULATE R FOR GRAPHING AS INCREASE NUMBER OF SIMULATIONS
%BG section 2 for graphical monitoring
if bat>1; disp(' Look at convergence graphs. Press key to proceed...');
for j=1:bat
    clear draws_bat s2 series mu W Bn Wi Bni Vi
    s2 = zeros(kk,kk);
    for i=1:m; series = draws(1:b*j,kk*(i-1)+1:kk*i);
        s2 = s2 + cov(series); mu(i,:)=mean(series);
    end
    W = s2/m; Bn = cov(mu); Wi = diag(W)'; Bni = diag(Bn)'; 
    Vi = ((b*j-1)/(b*j))*Wi + Bni;
    Ri(j,:) = sqrt(Vi./Wi); sqVi(j,:) = sqrt(Vi); sqWi(j,:) = sqrt(Wi);
    axi(j,1)=b*j;   %axis for plots
end
%plotting
if kk<=5
    figure
    for i=1:kk
        subplot(kk,2,2*i-1), plot(axi,Ri(:,i),'-or'), title('Ri');
        subplot(kk,2,2*i), plot(axi,sqVi(:,i),'-g',axi,sqWi(:,i),'-m'),
        title('Root Between and Within variances');
    end
elseif kk>5
    figure
    for i=1:5
        subplot(5,2,2*i-1), plot(axi,Ri(:,i),'-or'), title('Ri first 5 parameters');
        subplot(5,2,2*i), plot(axi,sqVi(:,i),'-g',axi,sqWi(:,i),'-m')
        title('Between and Within squared-root variances');
    end
    if kk<=10
        figure
        for i=6:kk
            subplot(kk-5,2,2*(i-5)-1), plot(axi,Ri(:,i),'-or'), title('Ri');
            subplot(kk-5,2,2*(i-5)), plot(axi,sqVi(:,i),'-g',axi,sqWi(:,i),'-m')
            title('Between and Within squared-root variances');
        end
    elseif kk>10
        figure
        for i=6:10
            subplot(5,2,2*(i-5)-1), plot(axi,Ri(:,i),'-or'), title('Ri, parameters 6-10');
            subplot(5,2,2*(i-5)), plot(axi,sqVi(:,i),'-g',axi,sqWi(:,i),'-m'),
            title('Between and Within squared-root variances');
        end
        if kk<=15
            figure
            for i=11:kk
                subplot(kk-10,2,2*(i-10)-1), plot(axi,Ri(:,i),'-or'), title('Ri');
                subplot(kk-10,2,2*(i-10)), plot(axi,sqVi(:,i),'-g',axi,sqWi(:,i),'-m'),
                title('Between and Within squared-root variances');
            end
        elseif kk>15
            figure
            for i=11:15
                subplot(5,2,2*(i-10)-1), plot(axi,Ri(:,i),'-or'), title('Ri, parameters 11-15');
                subplot(5,2,2*(i-10)), plot(axi,sqVi(:,i),'-g',axi,sqWi(:,i),'-m'),
                title('Between and Within squared-root variances');
            end
            if kk<=20
                figure
                for i=16:kk
                    subplot(kk-15,2,2*(i-15)-1), plot(axi,Ri(:,i),'-or'), title('Ri');
                    subplot(kk-15,2,2*(i-15)), plot(axi,sqVi(:,i),'-g',axi,sqWi(:,i),'-m'),
                    title('Between and Within squared-root variances');
                end
            elseif kk>20
                disp('Can only display graphs for k<=20!!!!');
            end
        end
    end
end
pause; close all;
end
disp('*********************************************************')